'use strict';

const electron = require('electron');
const path = require('path');

const eApp = electron.app;
const eglobalShortcut = electron.globalShortcut;
const eMenu = electron.Menu;
const BrowserWindow = electron.BrowserWindow;

const ipcMain = electron.ipcMain;
const childProcess = require('child_process');

let options = {};
options.url = process.argv[2];
options.listen = process.argv[3];

// ------------- RENDERER PROCESS MONITORING ----------------
const ENABLE_RENDERER_RELOAD_RESTART = true;
const ENABLE_RENDERER_MONITORING_LOG = false;

// Memory threshold
const MEMORY_THRESHOLD_WITH_SCREENSAVER = 40;
const MEMORY_THRESHOLD_WITHOUT_SCREENSAVER = 60;
// Restart if reload threshold has been reached
const RELOAD_THRESHOLD = 3;

let reloadCounter = 0;
let createWindowCounter = 0;
let isScreenSaverActivate = false;
// ---------------------------------------------------------

let mainWindow = null;
let appLocked = true;

let createWindow = () => {
	let CREPassword = null;

	mainWindow = new BrowserWindow({
		fullscreen: false,		//Attention bug sous Windows si True
		center: true,
		alwaysOnTop: true,
		autoHideMenuBar: true,
		useContentSize: true,
		webPreferences: {
			nodeIntegration: false,
			devTools: true,
			preload: path.join(__dirname, 'preload.js')
		}
	});

	mainWindow.loadURL('http://localhost:' + options.listen);

	// Pour le lancement en plein écran
	// fullscreen: true dans le constructeur bug sous Windows
	// alwaysOnTop property is bugged if using it from the options object in
	// BrowserWindow constructor. Setting it directly after works.
	mainWindow.setAlwaysOnTop(true, 'pop-up-menu', 2);
	mainWindow.setFullScreen(true);

	// Start xcursor app to hide cursor
	childProcess.exec(process.env.I4GEN + '/xcursor &');

	//Evènement qui arrive avant le Closed
	//e.preventDefault(); annule l'évènement
	mainWindow.on('close', () => {
	});

	mainWindow.on('closed', function onClosed() {
		mainWindow = null;
	});

	// Pour empécher l'ouverture de nouvelles fenêtres en faisant
	// Shift ou Ctrl + clique sur un lien
	mainWindow.webContents.on('new-window', (event) => {
		event.preventDefault();
	});

	// Pour ne pas avoir de menu par défaut et aussi aucun raccourci ! Aléluaia :)
	eMenu.setApplicationMenu(null);

	// template du menu que l'on affiche aprés saisi du mot de passe
	const template = [
		{
			label: 'Browser',
			submenu: [
				{
					label: 'Toggle fullscreen',
					accelerator: 'F11',
					role: 'togglefullscreen'
				},
				{
					label: 'Open dev log',
					accelerator: 'F12',
					role: 'toggledevtools'
				}
			]
		},
		{
			label: 'Support',
			submenu: [
				{
					label: 'Open terminal',
					accelerator: 'CTRL+T',
					click: () => {
						//childProcess.exec('mate-terminal --active --working-directory=' + process.env.I4GEN + ' -e "bash -c \'setxkbmap fr; exec /bin/bash\'"');
						childProcess.exec('mate-terminal --active --working-directory=/home/guest -e "bash -c \'su guest; exec /bin/bash\'"');
					}
				},
				// {
				// 	label: 'Show disk usage',
				// 	click: () => {
				// 		childProcess.exec('mate-terminal --tab --active --title "Disk Usage" -e "bash -c \'du -h --max-depth 2 \'' + process.env.I4GEN + '\';\'exec /bin/bash\'"');
				// 	}
				// },
				{
					label: 'View htop',
					click: () => {
						childProcess.exec('mate-terminal --full-screen -e "bash -c \'htop\';\'exec /bin/bash\'"');
					}
				},
				{
					label: 'List i4Gen processes',
					click: () => {
						childProcess.exec('mate-terminal --tab --active --title "List pm2 processes" -e "bash -c \'pm2 list\';\'exec /bin/bash\'"');
					}
				},
				{
					label: 'Restart client only',
					accelerator: 'F5',
					role: 'reload'
					// click: () => {
					// 	childProcess.exec('pm2 restart i4Gen_client');
					// }
				},
				{
					label: 'Restart server only',
					click: () => {
						childProcess.exec('pm2 restart i4Gen_server');
					}
				},
				{
					label: 'Restart updater only',
					click: () => {
						childProcess.exec('pm2 restart i4Gen_updater');
					}
				},
				{
					label: 'Stop server only',
					click: () => {
						childProcess.exec('pm2 stop i4Gen_server');
					}
				},
				{
					label: 'Close i4Gen',
					accelerator: 'ALT+F4',
					click: () => {
						childProcess.exec(process.env.I4GEN + '/stop_i4Gen.sh');
					}
				}
			]
		},
		{
			label: 'Logs',
			submenu: [
				{
					label: 'Show logs combined server',
					accelerator: 'CTRL+L',
					click: () => {
						childProcess.exec('mate-terminal --tab --active --title "Logs server output" -e "bash -c \'tail -f \'' + process.env.I4GEN + '\'/logs/server.combined-0.log\';\'exec /bin/bash\'"');
					}
				},
				{
					label: 'Show logs combined client',
					click: () => {
						childProcess.exec('mate-terminal --tab --active --title "Logs server output" -e "bash -c \'tail -f \'' + process.env.I4GEN + '\'/logs/client.combined-2.log\';\'exec /bin/bash\'"');
					}
				},
				{
					label: 'Show logs combined updater',
					click: () => {
						childProcess.exec('mate-terminal --tab --active --title "Logs server output" -e "bash -c \'tail -f \'' + process.env.I4GEN + '\'/logs/updater.combined-1.log\';\'exec /bin/bash\'"');
					}
				}
			]
		}
	];
	const menu = eMenu.buildFromTemplate(template);

	mainWindow.on('enter-full-screen', function onEnterFullScreen() {
		//console.log('enter-full-screen');

		eglobalShortcut.register('CmdOrCtrl+I', () => {
			console.log('CmdOrCtrl+I is pressed from enter-full-screen');
			CREPassword = 'i';
		});

		eglobalShortcut.register('CmdOrCtrl+F', () => {
			console.log('CmdOrCtrl+F is pressed from enter-full-screen');
			if (CREPassword === 'i') {
				CREPassword += 'f';
			} else {
				CREPassword = '';
			}
		});

		eglobalShortcut.register('CmdOrCtrl+O', () => {
			console.log('CmdOrCtrl+O is pressed from enter-full-screen');
			if (CREPassword === 'if') {
				CREPassword += 'o';
			} else {
				CREPassword = '';
			}
		});

		eglobalShortcut.register('CmdOrCtrl+R', () => {
			console.log('CmdOrCtrl+R is pressed from enter-full-screen');
			if (CREPassword === 'ifo') {
				CREPassword += 'r';
			} else {
				CREPassword = '';
			}
		});

		eglobalShortcut.register('CmdOrCtrl+G', () => {
			console.log('CmdOrCtrl+G is pressed from enter-full-screen');
			if (CREPassword === 'ifor') {
				CREPassword += 'g';
			} else {
				CREPassword = '';
			}
		});

		eglobalShortcut.register('CmdOrCtrl+E', () => {
			console.log('CmdOrCtrl+E is pressed from enter-full-screen');
			if (CREPassword === 'iforg') {
				CREPassword += 'e';
			} else {
				CREPassword = '';
			}
		});

		// On pourrait faire un RAZ de CREPassword sur toutes les autres lettres du clavier
		// pour augmenter la sécurité

		eglobalShortcut.register('CmdOrCtrl+N', () => {
			console.log('CmdOrCtrl+N is pressed from enter-full-screen');
			if (CREPassword === 'iforge') {
				CREPassword += 'n';
			} else {
				CREPassword = '';
			}

			// Chaque saisie du mot de passe débloque si bloqué et inversement l'application
			// en reconstruisant le menu (déblocage) ou en le supprimant (blocage)
			if (CREPassword === 'iforgen') {
				if (appLocked === true) {
					//Construire le menu défini dans le template
					eMenu.setApplicationMenu(menu);
					//Rend visible le menu (ALT permet de l'afficher ou de le cacher)
					mainWindow.setMenuBarVisibility(true);
					//Kill xcursor app to show cursor
					childProcess.exec('killall xcursor');
					appLocked = false;
				} else {
					//Supprime le menu
					eMenu.setApplicationMenu(null);
					// Start xcursor app to hide cursor
					childProcess.exec(process.env.I4GEN + '/xcursor &');
					appLocked = true;
				}
			}
		});
	});

	mainWindow.on('leave-full-screen', function onLeaveFullScreen() {
		console.log('leave-full-screen');
		eglobalShortcut.unregisterAll();
	});
};

let startMemoryUsageTimeout = () => {
	let PIDRenderer = 0;
	let MEMORYRenderer = 0;

	// If screensaver change his state
	childProcess.exec('pgrep -f /usr/lib/mate-screensaver', (errorScreensaver, stdoutScreensaver, stderrScreensaver) => {
		if (errorScreensaver) {
			console.log(`pgrep -f /usr/lib/mate-screensaver error: ${errorScreensaver} / ${stderrScreensaver}`);
		}
		if (parseInt((stdoutScreensaver.split('\n').length - 1), 10) === 2) {
			//console.log('Screensaver ON');
			isScreenSaverActivate = true;
		} else {
			//console.log('Screensaver OFF');
			isScreenSaverActivate = false;
		}

		// Get PID Renderer process
		const pgrepRenderer = childProcess.exec('pgrep -f \'electron --type=renderer\'', (error, stdout, stderr) => {
			if (error) {
				console.log(`pgrep renderer error: ${error} / ${stderr}`);
			}

			//console.log('Command line PID:', pgrepRenderer.pid);

			let arrPgrepRenderer = stdout.split('\n');
			arrPgrepRenderer.splice(arrPgrepRenderer.length - 1, 1);

			for (let i = 0; i < arrPgrepRenderer.length; i += 1) {
				// Convert PID to int
				arrPgrepRenderer[i] = parseInt(arrPgrepRenderer[i], 10);

				// Find command line PID and remove it
				if (arrPgrepRenderer[i] === pgrepRenderer.pid) {
					arrPgrepRenderer.splice(i, 1);
				}
			}

			// PRINT LOG
			for (let i = 0; i < arrPgrepRenderer.length; i += 1) {
				//console.log('Renderer process PID:', arrPgrepRenderer[i]);
			}

			let nbrRendererProcess = parseInt(arrPgrepRenderer.length, 10);
			//console.log('nbrRendererProcess:', nbrRendererProcess);

			// If multi renderer process
			if (nbrRendererProcess > 1) {
				if (appLocked) {
					if (ENABLE_RENDERER_RELOAD_RESTART) {
						console.log(new Date().getDate() + '|' + new Date().toLocaleTimeString(), 'ERROR: MULTI RENDERER PROCESS => RESTART !!!');
					}
					reloadCounter = 0;
					createWindowCounter += 1;
					if (ENABLE_RENDERER_RELOAD_RESTART) {
						childProcess.exec('killall electron; killall electron');
					}
				}

			// If crash of renderer process
			} else if (nbrRendererProcess === 0) {
				console.log(new Date().getDate() + '|' + new Date().toLocaleTimeString(), 'ERROR: RENDERER PROCESS NOT RUNNING => RESTART !!!');
				createWindowCounter += 1;
				reloadCounter = 0;
				if (ENABLE_RENDERER_RELOAD_RESTART) {
					childProcess.exec('killall electron; killall electron');
				}

			// Else GET memory usage of renderer process
			} else {
				PIDRenderer = arrPgrepRenderer[0];
				childProcess.exec('ps -p ' + PIDRenderer + ' -o pmem | tail -1', (errorpmem, stdoutpmem, stderrpmem) => {
					if (errorpmem) {
						console.log(`ps -p error: ${errorpmem} / ${stderrpmem}`);
					}

					MEMORYRenderer = parseInt(stdoutpmem, 10);
					if (ENABLE_RENDERER_MONITORING_LOG) {
						console.log(new Date().getDate() + '|' + new Date().toLocaleTimeString(), '- MemoryUsage:', MEMORYRenderer + '% - (restart/reload):', createWindowCounter, '/', reloadCounter);
					}
					// If screensaver is running
					if (isScreenSaverActivate) {
						// If MEMORY_THRESHOLD_WITH_SCREENSAVER has been reached => restart
						if (MEMORYRenderer >= MEMORY_THRESHOLD_WITH_SCREENSAVER) {
							console.log(new Date().getDate() + '|' + new Date().toLocaleTimeString(), 'WARNING MEMORY WITH SCREENSAVER => RESTART !');
							reloadCounter = 0;
							createWindowCounter += 1;
							if (ENABLE_RENDERER_RELOAD_RESTART) {
								childProcess.exec('killall electron; killall electron');
							}
						}
					// Else if MEMORY_THRESHOLD_WITHOUT_SCREENSAVER reached
					} else if (MEMORYRenderer >= MEMORY_THRESHOLD_WITHOUT_SCREENSAVER) {
						// After RELOAD_THRESHOLD x reloads => restart
						if (reloadCounter >= RELOAD_THRESHOLD) {
							console.log(new Date().getDate() + '|' + new Date().toLocaleTimeString(), 'WARNING MEMORY WITHOUT SCREENSAVER => RESTART !');
							reloadCounter = 0;
							createWindowCounter += 1;
							if (ENABLE_RENDERER_RELOAD_RESTART) {
								childProcess.exec('killall electron; killall electron');
							}
						// Else reload
						} else {
							console.log(new Date().getDate() + '|' + new Date().toLocaleTimeString(), 'WARNING MEMORY WITHOUT SCREENSAVER => RELOAD !');
							reloadCounter += 1;
							if (ENABLE_RENDERER_RELOAD_RESTART) {
								mainWindow.reload();
							}
						}
					}
				});
			}
		});
	});
	setTimeout(startMemoryUsageTimeout, 5000);
};

setTimeout(startMemoryUsageTimeout, 1000);

// Quit when all windows are closed.
eApp.on('window-all-closed', function onWindowAllClosed() {
	//Pour arrêter le processus sur fermeture de la fenêtre
	//Sinon il continue à tourner et on ne peut alors pas relancer l'interface
	//Il faut alors killer le processus
	eApp.quit();
});

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
eApp.on('ready', createWindow);

eApp.on('activate', function activate() {
	// On OS X it's common to re-create a window in the app when the
	// dock icon is clicked and there are no other windows open.
	if (mainWindow === null) {
		createWindow();
	}
});

// Listen for async message from renderer process
ipcMain.on('updater', (event, arg) => {
	// Print 1
	console.log(arg);

	if (arg === 1) {
		mainWindow.loadURL('http://localhost:3001');
		childProcess.exec(process.env.I4GEN + '/lastupdatefiles/updater.sh');
	}

	// Reply on async message from renderer process
	//event.sender.send('async-reply', 2);
});

// Listen for async message from renderer process
ipcMain.on('restart', (event, arg) => {
	//console.log('restart');
	childProcess.exec('shutdown -r now;');
});

// Listen for async message from renderer process
ipcMain.on('log', (event, arg) => {
	console.log(new Date().getDate() + '|' + new Date().toLocaleTimeString(), '- RENDERER -', arg);
});
