'use strict'

let stampit = require('stampit')
let Promise = require('bluebird')

module.exports = stampit()
  .init(function () {
    let init = function () {
      this.addResponseHandler(21, onResponse)
    }.bind(this)

    let onResponse = function (pdu, request) {
      this.log.debug('handling file records response.')
      this.log.debug('on write file records got PDU: ' + JSON.stringify(pdu) + ' pdu.length:' + pdu.length)
      this.log.debug('request: ' + JSON.stringify(request))

      if (pdu.length < 5) {
        request.defer.reject()
        return
      }

      try {
        let fc = pdu.readUInt8(0)
        if (fc !== 21) {
          request.defer.reject()
          return
        }

        let startAddress = pdu.readUInt16BE(1)
        let quantity = pdu.readUInt16BE(3)
        let resp = {
          fc: fc,
          startAddress: startAddress,
          quantity: quantity
        }

        request.defer.resolve(resp)
      } catch(e) {
          console.log('writeFileRecord error', e)
          console.log('pdu', pdu)
          request.defer.reject()
      }
    }.bind(this)

    this.writeFileRecord = function (fileNumber, recordNumber, recordLength, text) {
      return new Promise(function (resolve, reject) {

        let fc = 21
        let byteCount = 7 + text.length /* Pour la lecture d'un enregistrement */
        let refType = 6
        let basePdu = Buffer.allocUnsafe(9)
        let pdu

        basePdu.writeUInt8(fc)
        basePdu.writeUInt8(byteCount, 1)
        basePdu.writeUInt8(refType, 2)
        basePdu.writeUInt16BE(fileNumber, 3)
        basePdu.writeUInt16BE(recordNumber, 5)
        basePdu.writeUInt16BE(recordLength, 7)

        if (text instanceof Buffer) {
          console.log('text instanceof Buffer');
          if (text.length / 2 > 0x007b) {
            reject()
          }

          pdu = Buffer.concat([basePdu, text])

        } else if (text instanceof Array) {
          console.log('text instanceof Array');
          if (text.length > 0x007b) {
            reject()
            return
          }

          let payloadPdu = Buffer.allocUnsafe(text.length)

          for (let i = 0; i < text.length; i += 1) {
            payloadPdu.writeUInt8(text[i].charCodeAt(0), i)
          }

          pdu = Buffer.concat([basePdu, payloadPdu])

        } else {
          console.log('reject writeFileRecord');
          reject()
          return
        }

        this.queueRequest(fc, pdu, { resolve: resolve, reject: reject })
      }.bind(this))
    }

    init()
  })
